#===========================================================================
#      _    _          __  _   
#     | |  | |        / _|| |        Wafl Project
#     | |  | |  __ _ | |_ | |        -----------------------------
#     | |/\| | / _` ||  _|| |        Wafl implementation and tools
#     \  /\  /| (_| || |  | |            
#      \/  \/  \__,_||_|   \_\       Copyright (C) 1997-2023, Saša Malkov
#                                  
#===========================================================================
# DetectPlatform.cmake
#===========================================================================

#----------------------------------------------
# DETECT_PLATFORM
# All relevant variables will be included in cmake_config.h
#----------------------------------------------
# Detect WORD size
include( CheckTypeSize )
check_type_size( void* 				WAFL_BUILD_VOID_PTR_SIZE	LANGUAGE CXX ) 
check_type_size( short 				WAFL_BUILD_SIZEOF_SHORT		LANGUAGE CXX ) 
check_type_size( int 				WAFL_BUILD_SIZEOF_INT		LANGUAGE CXX ) 
check_type_size( "long int" 		WAFL_BUILD_SIZEOF_LONG 		LANGUAGE CXX ) 
check_type_size( "long long int" 	WAFL_BUILD_SIZEOF_LONGLONG 	LANGUAGE CXX )
if( WAFL_BUILD_VOID_PTR_SIZE EQUAL 8 )
	set( WAFL_BUILD_64BIT "1" CACHE INTERNAL "" FORCE )	
elseif( WAFL_BUILD_VOID_PTR_SIZE EQUAL 4 )
	set( WAFL_BUILD_32BIT "1" CACHE INTERNAL "" FORCE )	
else()
	message( FATAL_ERROR
		"The project is not prepared for platforms other than 32bit and 64bit!" )
endif()


#----------------------------------------------
# Detect WORD size integer type
if( WAFL_BUILD_VOID_PTR_SIZE EQUAL WAFL_BUILD_SIZEOF_LONGLONG )
	set( WAFL_BUILD_PTR_INT "long long" CACHE INTERNAL "" FORCE )
elseif( WAFL_BUILD_VOID_PTR_SIZE EQUAL WAFL_BUILD_SIZEOF_LONG )
	set( WAFL_BUILD_PTR_INT "long" CACHE INTERNAL "" FORCE )
elseif( WAFL_BUILD_VOID_PTR_SIZE EQUAL WAFL_BUILD_SIZEOF_INT )
	set( WAFL_BUILD_PTR_INT "int" CACHE INTERNAL "" FORCE )
elseif( WAFL_BUILD_VOID_PTR_SIZE EQUAL WAFL_BUILD_SIZEOF_SHORT )
	set( WAFL_BUILD_PTR_INT "short" CACHE INTERNAL "" FORCE )
else()
	message( FATAL_ERROR
		"The project cannot determine a word size!" )
endif()
set( WAFL_BUILD_PTR_UINT "unsigned ${WAFL_BUILD_PTR_INT}" CACHE INTERNAL "" FORCE )


#----------------------------------------------
# Detect BIG/LITTLE ENDIAN
#----------------------------------------------
include( TestBigEndian )
test_big_endian( WAFL_BUILD_IS_BIG_ENDIAN )
if ( WAFL_BUILD_IS_BIG_ENDIAN )
	set( WAFL_BUILD_IS_LITTLE_ENDIAN 0 CACHE INTERNAL "" FORCE )
else ()
	set( WAFL_BUILD_IS_LITTLE_ENDIAN 1 CACHE INTERNAL "" FORCE )
endif ()


# Detect target platform
if( CMAKE_GENERATOR STREQUAL "Unix Makefiles" )
	set( WAFL_BUILD_TARGET_LINUX "1" CACHE INTERNAL "" FORCE )	
	set( WAFL_BUILD_UNIX "1" CACHE INTERNAL "" FORCE )
	set( WAFL_PLATFORM_CODE "uxmkfl" CACHE INTERNAL "" FORCE )
elseif( CMAKE_GENERATOR STREQUAL "MinGW Makefiles" )
	set( WAFL_BUILD_TARGET_MINGW "1" CACHE INTERNAL "" FORCE )	
	set( WAFL_BUILD_MINGW "1" CACHE INTERNAL "" FORCE )
	if( WAFL_BUILD_VOID_PTR_SIZE EQUAL 8 )
		set( WAFL_BUILD_MINGW64 "1" CACHE INTERNAL "" FORCE )
	else()
		set( WAFL_BUILD_MINGW32 "1" CACHE INTERNAL "" FORCE )
	endif()
	set( WAFL_PLATFORM_CODE "mngwmkfl" CACHE INTERNAL "" FORCE )
elseif( CMAKE_GENERATOR MATCHES "Visual Studio .*" )
	set( WAFL_BUILD_TARGET_WINDOWS "1" CACHE INTERNAL "" FORCE )	
	set( WAFL_BUILD_MSVC "1" CACHE INTERNAL "" FORCE )	
	set( WAFL_BUILD_MSVC_IDE "${MSVC_IDE}" CACHE INTERNAL "" FORCE )	
	set( WAFL_BUILD_MSVC_VERSION "${MSVC_VERSION}" CACHE INTERNAL "" FORCE )	
	if( CMAKE_GENERATOR MATCHES ".* Win64" )
		set( WAFL_BUILD_TARGET_WINDOWS64 "1" CACHE INTERNAL "" FORCE )	
		set( WAFL_PLATFORM_CODE "win64" CACHE INTERNAL "" FORCE )
	elseif( CMAKE_GENERATOR_PLATFORM MATCHES "x64" )
		set( WAFL_BUILD_TARGET_WINDOWS64 "1" CACHE INTERNAL "" FORCE )	
		set( WAFL_PLATFORM_CODE "win64" CACHE INTERNAL "" FORCE )
	elseif( CMAKE_GENERATOR_PLATFORM MATCHES "X64" )
		set( WAFL_BUILD_TARGET_WINDOWS64 "1" CACHE INTERNAL "" FORCE )	
		set( WAFL_PLATFORM_CODE "win64" CACHE INTERNAL "" FORCE )
	else()
		set( WAFL_BUILD_TARGET_WINDOWS32 "1" CACHE INTERNAL "" FORCE )	
		set( WAFL_PLATFORM_CODE "win32" CACHE INTERNAL "" FORCE )
	endif()
else()
	set( WAFL_BUILD_BORLAND "${BORLAND}" CACHE INTERNAL "" FORCE )	
	set( WAFL_BUILD_CYGWIN "${CYGWIN}" CACHE INTERNAL "" FORCE )	
	set( WAFL_PLATFORM_CODE "undef" CACHE INTERNAL "" FORCE )
endif()


#----------------------------------------------
# Set Integral Platform Id
set( WAFL_PLATFORM_IDSTRING "${CMAKE_CXX_COMPILER_ID}" CACHE INTERNAL "" FORCE )
if( CMAKE_CXX_COMPILER_ID STREQUAL "MSVC" )
	set( WAFL_PLATFORM_IDSTRING "${WAFL_PLATFORM_IDSTRING}_${MSVC_TOOLSET_VERSION}" CACHE INTERNAL "" FORCE )
else()
	set( WAFL_PLATFORM_IDSTRING "${WAFL_PLATFORM_IDSTRING}_${CMAKE_CXX_COMPILER_VERSION}" CACHE INTERNAL "" FORCE )
endif()
set( WAFL_PLATFORM_IDSTRING "${WAFL_PLATFORM_IDSTRING}_${WAFL_PLATFORM_CODE}_${CMAKE_SYSTEM_PROCESSOR}_${WAFL_BUILD_VOID_PTR_SIZE}_${WAFL_BUILD_IS_LITTLE_ENDIAN}_CPP${CMAKE_CXX_STANDARD}_${CMAKE_BUILD_TYPE}" CACHE INTERNAL "" FORCE )


#----------------------------------------------
