#===========================================================================
#      _    _          __  _   
#     | |  | |        / _|| |        Wafl Project
#     | |  | |  __ _ | |_ | |        -----------------------------
#     | |/\| | / _` ||  _|| |        Wafl implementation and tools
#     \  /\  /| (_| || |  | |            
#      \/  \/  \__,_||_|   \_\       Copyright (C) 1997-2023, Saša Malkov
#                                  
#===========================================================================
# SetCompilerOptions.cmake
#===========================================================================

#----------------------------------------------
# COMPILER OPTIONS
#----------------------------------------------
# Clang
if ("${CMAKE_CXX_COMPILER_ID}" STREQUAL "Clang")
	#	add_definitions(-DCONFIG_COMPILER_CLANG)

# GCC
elseif ("${CMAKE_CXX_COMPILER_ID}" STREQUAL "GNU")
	# If compiling to ASM, make it more verbose
	# string( APPEND CMAKE_CXX_FLAGS " -fverbose-asm")
	
	# Generate RTL files for the code analysis
	#-fdump-cpp-all -fdump-tree-all
	
	# FOR DEPENDENCY TREE!!!
	# string( APPEND CMAKE_CXX_FLAGS " -fdump-rtl-expand -fdump-class-hierarchy")
	
	# GCC threading options - Not the same for Linux and MINGW
	if(${WAFL_BUILD_MINGW})
		add_link_options( -pthread ) 
	else()
		add_link_options( -pthread -ldl ) 
	endif()

	add_compile_options(
		$<$<COMPILE_LANGUAGE:CXX>:-Wsuggest-override>	# Report warning if not override is used for virtual overrides
	)
	
	if( "${CMAKE_BUILD_TYPE}" STREQUAL "Release" )
		add_link_options(
			-s					# strip debug info
		)
	endif()
	

	# The following options should make the code smaller,
	# but they do almost nothing.
	# add_compile_options(
	# 	$<$<COMPILE_LANGUAGE:CXX>:-ffunction-sections>	# Collect data on unused functions
	# 	$<$<COMPILE_LANGUAGE:CXX>:-fdata-sections>		# Collect data on unused data
	# 	$<$<COMPILE_LANGUAGE:CXX>:-Wl,--gcc-sections>	# Send to linker option to remove unused sections
	# )

	# The following options perform Link-Time-Optimization,
	# but only reduce code size for a little and make it slower (!!!)
	# add_compile_options(
	# 	$<$<COMPILE_LANGUAGE:CXX>:-flto=auto>			# Link time optimizations
	# )

	# Additional options for Debug build (!!! not for ARM !!!)
# todo: There is a bug with sanitizers from march/2024
# that falls in "AddressSanitizer:DEADLYSIGNAL" deadly loop.
#	if ( NOT "${CMAKE_SYSTEM_PROCESSOR}" STREQUAL "aarch64")
#		string( APPEND CMAKE_CXX_FLAGS_DEBUG " -fsanitize=address" )
#	endif()

# Intel
elseif ("${CMAKE_CXX_COMPILER_ID}" STREQUAL "Intel")
	#	add_definitions(-DCONFIG_COMPILER_INTEL)
	#	add_compile_options( "-fno-rtti" )

# MSVC
elseif ("${CMAKE_CXX_COMPILER_ID}" STREQUAL "MSVC")
	# Iterator debugging makes result stack unusable.
	# It does not allow constructions like &*end() and similar, and we need this.
	# So we disable it (=0)
	add_definitions( -D_ITERATOR_DEBUG_LEVEL=0 )
	
	# Unicode and UTF-8 compile options only bring some problems...
	# Use the following pragma instead, in all files with UTF-8:
	# 		#ifdef MSVC
	# 			#pragma execution_character_set("utf-8")
	# 		#endif
	#add_definitions( -D_UNICODE )
	#add_definitions( -DUNICODE )
	#add_compile_options( /utf-8 )

	# Remove linker warning for linking nodebug info lib objs in debug mode.
	#string( APPEND CMAKE_EXE_LINKER_FLAGS " /IGNORE:4099")

	add_compile_options(
		$<$<COMPILE_LANGUAGE:C,CXX>:/MP>		# Enable multithread compilation
		$<$<COMPILE_LANGUAGE:C,CXX>:/w14263>	# Report warning if not override is used for virtual overrides
		$<$<COMPILE_LANGUAGE:C,CXX>:/wd5051>	# Disable warning for not using [[unlikely]] before c++20
	)
	add_link_options(
		/IGNORE:4221		# Remove empty obj file linker warning
		/IGNORE:4099		# Remove linker warning for linking nodebug info lib objs in debug mode.
	)

	if( "${CMAKE_BUILD_TYPE}" STREQUAL "Debug" )
		add_compile_options(
			$<$<COMPILE_LANGUAGE:CXX>:/MTd>		# Force static CRT linkage
		)
	else()
		add_compile_options(
			$<$<COMPILE_LANGUAGE:CXX>:/MT>		# Force static CRT linkage
			$<$<COMPILE_LANGUAGE:CXX>:/EHsc>	# Turn standard exception handling on

			$<$<COMPILE_LANGUAGE:C,CXX>:/Oi>	# Optimizations
			$<$<COMPILE_LANGUAGE:C,CXX>:/Ot>	# Optimizations
			$<$<COMPILE_LANGUAGE:C,CXX>:/Ob2>	# Optimizations
			$<$<COMPILE_LANGUAGE:C,CXX>:/GL>	# Whole program optimizations
		)
		add_link_options(
			/LTCG				# Whole program optimizations
			/INCREMENTAL:NO		# Disable incremental link
		)
	endif()

endif()

#----------------------------------------------
