#===========================================================================
#      _    _          __  _   
#     | |  | |        / _|| |        Wafl Project
#     | |  | |  __ _ | |_ | |        -----------------------------
#     | |/\| | / _` ||  _|| |        Wafl implementation and tools
#     \  /\  /| (_| || |  | |            
#      \/  \/  \__,_||_|   \_\       Copyright (C) 1997-2023, Saša Malkov
#                                  
#===========================================================================
# WaflLibraryCore.cmake
#===========================================================================

# IMPORTANT:
#	Assume WAFLCORE_PATH is set to the waflcore root path,
#	which contains 

# Detect platform
include( DetectPlatform )
# Add functions
include( AddWindowsResources )
# Set compiler options
include( SetCompilerOptions )

message( STATUS "WAFL_PLATFORM_IDSTRING = ${WAFL_PLATFORM_IDSTRING}")


#----------------------------------------------
# We already use "lib" prefix explicitly for some kinds of subprojects, 
# so do not add this again!
set(CMAKE_STATIC_LIBRARY_PREFIX "")
set(CMAKE_SHARED_MODULE_PREFIX "")
set(CMAKE_SHARED_LIBRARY_PREFIX "")

# Organize projects in folders (in VS, for example)
set_property( GLOBAL PROPERTY USE_FOLDERS ON)


#----------------------------------------------
# BUILD CONFIG
#----------------------------------------------
# if not set by build command, set build type
if(NOT CMAKE_BUILD_TYPE)
	set(CMAKE_BUILD_TYPE "${default_build_type}")
endif()

# make an additional helper def...
string(TOUPPER ${CMAKE_BUILD_TYPE} WAFL_BUILD_TYPE)
set("WAFL_BUILD_TYPE_${WAFL_BUILD_TYPE}" 1)

# CMAKE CONFIG
#	...

# C++17 Required
set(CMAKE_CXX_STANDARD 17)
set(CMAKE_CXX_STANDARD_REQUIRED ON)
#set(CMAKE_CXX_EXTENSIONS OFF)

# To prepare libs source for build of both static programs and dynamic libraries
set(CMAKE_POSITION_INDEPENDENT_CODE ON)


#----------------------------------------------
# BINARY TARGET DIRECTORIES
#----------------------------------------------
set( CMAKE_RUNTIME_OUTPUT_DIRECTORY 				${CMAKE_BINARY_DIR}/_${CMAKE_BUILD_TYPE}/bin)
set( CMAKE_RUNTIME_OUTPUT_DIRECTORY_DEBUG 			${CMAKE_BINARY_DIR}/_Debug/bin)
set( CMAKE_RUNTIME_OUTPUT_DIRECTORY_RELEASE 		${CMAKE_BINARY_DIR}/_Release/bin)
set( CMAKE_RUNTIME_OUTPUT_DIRECTORY_RELWITHDEBINFO	${CMAKE_BINARY_DIR}/_RelWithDebInfo/bin)

set( CMAKE_LIBRARY_OUTPUT_DIRECTORY 				${CMAKE_BINARY_DIR}/_${CMAKE_BUILD_TYPE}/bin)
set( CMAKE_LIBRARY_OUTPUT_DIRECTORY_DEBUG 			${CMAKE_BINARY_DIR}/_Debug/bin)
set( CMAKE_LIBRARY_OUTPUT_DIRECTORY_RELEASE 		${CMAKE_BINARY_DIR}/_Release/bin)
set( CMAKE_LIBRARY_OUTPUT_DIRECTORY_RELWITHDEBINFO	${CMAKE_BINARY_DIR}/_RelWithDebInfo/bin)

set( CMAKE_ARCHIVE_OUTPUT_DIRECTORY 				${CMAKE_BINARY_DIR}/_${CMAKE_BUILD_TYPE}/lib)
set( CMAKE_ARCHIVE_OUTPUT_DIRECTORY_DEBUG 			${CMAKE_BINARY_DIR}/_Debug/lib)
set( CMAKE_ARCHIVE_OUTPUT_DIRECTORY_RELEASE 		${CMAKE_BINARY_DIR}/_Release/lib)
set( CMAKE_ARCHIVE_OUTPUT_DIRECTORY_RELWITHDEBINFO	${CMAKE_BINARY_DIR}/_RelWithDebInfo/lib)

set( WAFL_BUILD_OUTPUT_DOC_DIR						${CMAKE_BINARY_DIR}/_${CMAKE_BUILD_TYPE}/doc)

message(STATUS "Target Build Directory:     CMAKE_RUNTIME_OUTPUT_DIRECTORY = ${CMAKE_RUNTIME_OUTPUT_DIRECTORY}")

#----------------------------------------------
# INSTALL DIRECTORIES
#----------------------------------------------
# We do not want this to be installed directly to the program directories.
# Instead, we will just make a collection of files, ready to move.
set( WAFL_INSTALL_ROOT 		${CMAKE_BINARY_DIR}/_install )
set( WAFL_INSTALL_BIN 		${WAFL_INSTALL_ROOT}/bin )
set( WAFL_INSTALL_TEST 		${WAFL_INSTALL_ROOT}/test )
set( WAFL_INSTALL_LIB 		${WAFL_INSTALL_ROOT}/lib )
set( WAFL_INSTALL_INCLUDE	${WAFL_INSTALL_ROOT}/include )

file(MAKE_DIRECTORY ${WAFL_INSTALL_ROOT} )
file(MAKE_DIRECTORY ${WAFL_INSTALL_BIN} )
file(MAKE_DIRECTORY ${WAFL_INSTALL_TEST} )
file(MAKE_DIRECTORY ${WAFL_INSTALL_LIB} )
file(MAKE_DIRECTORY ${WAFL_INSTALL_INCLUDE} )

message(STATUS "WAFL_INSTALL_ROOT = ${WAFL_INSTALL_ROOT}")



#----------------------------------------------
# ENABLE ASSEMBLER
#----------------------------------------------
if( WAFL_BUILD_TARGET_WINDOWS )
	enable_language( ASM_MASM )
	message(STATUS "Asm supported: ${CMAKE_ASM_MASM_COMPILER}")
endif()
  
  
  
#----------------------------------------------
# EXTERN LIBS
#----------------------------------------------
# catch2
add_library( extern::catch2 IMPORTED INTERFACE )
target_link_libraries( extern::catch2 INTERFACE
	CONAN_PKG::catch2 
)
	
#[[
#Alternative settings
	find_package(catch2 REQUIRED)
	if( catch2_FOUND )
		add_library( extern::catch2 IMPORTED INTERFACE )
		set_target_properties( extern::catch2 PROPERTIES
			INTERFACE_INCLUDE_DIRECTORIES		"${CATCH_INCLUDE_DIRS}"
		)
		target_link_libraries( extern::catch2 INTERFACE
			catch2::catch2 
		)
	endif()
]]



#----------------------------------------------
# WaflCore
if( NOT EXISTS "${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}" )
	message( FATAL_ERROR "Wafl Core libraries not found for platform ${WAFL_PLATFORM_IDSTRING}" )
elseif( NOT EXISTS "${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}/include/waflcore/libExtern.h" )
	message( FATAL_ERROR "Wafl Core include files missing for platform ${WAFL_PLATFORM_IDSTRING}" )
elseif( NOT EXISTS "${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}/include/waflcore/libExternBase.h" )
	message( FATAL_ERROR "Wafl Core include files missing for platform ${WAFL_PLATFORM_IDSTRING}" )
elseif( WAFL_BUILD_TARGET_WINDOWS AND NOT EXISTS "${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}/lib/libWaflCore.lib" )
	message( FATAL_ERROR "Wafl Core static libraries missing for platform ${WAFL_PLATFORM_IDSTRING}" )
elseif( NOT WAFL_BUILD_TARGET_WINDOWS AND NOT EXISTS "${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}/lib/libWaflCore.a" )
	message( FATAL_ERROR "Wafl Core static libraries missing for platform ${WAFL_PLATFORM_IDSTRING}" )
endif()

add_library( libWaflCore STATIC IMPORTED )
if( WAFL_BUILD_TARGET_WINDOWS )
	set_target_properties( libWaflCore PROPERTIES
		INTERFACE_INCLUDE_DIRECTORIES 		"${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}/include"
		IMPORTED_LOCATION					"${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}/lib/libWaflCore.lib"
	)
else()
	set_target_properties( libWaflCore PROPERTIES
		INTERFACE_INCLUDE_DIRECTORIES 		"${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}/include"
		IMPORTED_LOCATION					"${WAFLCORE_PATH}/${WAFL_PLATFORM_IDSTRING}/lib/libWaflCore.a"
	)
endif()


#----------------------------------------------
